package studentmanagement.api.rest.controller.administrator;

import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import studentmanagement.model.API.API;
import studentmanagement.model.dto.create.AdminRegisterRequest;
import studentmanagement.model.dto.create.AdminUpsertRequest;
import studentmanagement.model.dto.create.RoleChangeRequest;
import studentmanagement.model.dto.find.StudentCourseIdRequest;
import studentmanagement.model.dto.find.TeacherCourseIdRequest;
import studentmanagement.model.dto.response.AdminUpsertResponse;
import studentmanagement.model.dto.response.LoginResponse;
import studentmanagement.model.dto.response.RoleChangeResponse;
import studentmanagement.model.dto.response.StudentCourseResponse;
import studentmanagement.model.dto.response.TeacherCourseResponse;
import studentmanagement.service.administrator.AdministratorService;
import studentmanagement.service.role.RoleChangeService;

@RestController
@RequestMapping(API.V1 + API.ADMIN)
@AllArgsConstructor
public class DefaultAdministratorController {

    @Autowired
    private final AdministratorService administratorService;

    @Autowired
    private final RoleChangeService roleChangeService;


    @PostMapping(API.REGISTER)
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<LoginResponse> registerAdmin(@Validated @RequestBody AdminRegisterRequest request) {
        return new ResponseEntity<>(administratorService.registerAdmin(request), HttpStatus.CREATED);
    }

    @PutMapping
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<AdminUpsertResponse> upsert(@Validated @RequestBody AdminUpsertRequest request) {
        return new ResponseEntity<>(administratorService.upsert(request), HttpStatus.ACCEPTED);
    }

    @PatchMapping(API.ENROLL)
    @PreAuthorize(API.ADMIN_TEACHER_AUTHORITY)
    public ResponseEntity<StudentCourseResponse> enroll(@Validated @RequestBody StudentCourseIdRequest request) {
        return new ResponseEntity<>(administratorService.enroll(request), HttpStatus.ACCEPTED);
    }

    @PatchMapping(API.DISMISS)
    @PreAuthorize(API.ADMIN_TEACHER_AUTHORITY)
    public ResponseEntity<StudentCourseResponse> dismiss(@Validated @RequestBody StudentCourseIdRequest request) {
        return new ResponseEntity<>(administratorService.dismiss(request), HttpStatus.ACCEPTED);
    }

    @PatchMapping(API.EMPLOY)
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<TeacherCourseResponse> employ(@Validated @RequestBody TeacherCourseIdRequest request) {
        return new ResponseEntity<>(administratorService.employ(request), HttpStatus.ACCEPTED);
    }

    @PatchMapping(API.FIRE)
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<TeacherCourseResponse> fire(@Validated @RequestBody TeacherCourseIdRequest request) {
        return new ResponseEntity<>(administratorService.fire(request), HttpStatus.ACCEPTED);
    }

    @PutMapping(API.ROLE_CHANGE)
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<RoleChangeResponse> changeRole(@Validated @RequestBody
                                                         RoleChangeRequest request) {
        return new ResponseEntity<>(roleChangeService.changeRole(request), HttpStatus.ACCEPTED);
    }
}