package studentmanagement.api.rest.controller.authentication;

import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import studentmanagement.model.API.API;
import studentmanagement.model.dto.create.AdminLoginRequest;
import studentmanagement.model.dto.create.StudentLoginRequest;
import studentmanagement.model.dto.create.StudentRegisterRequest;
import studentmanagement.model.dto.create.TeacherLoginRequest;
import studentmanagement.model.dto.create.TeacherRegisterRequest;
import studentmanagement.model.dto.response.LoginResponse;
import studentmanagement.service.authentication.AuthenticationService;
import studentmanagement.service.cookie.CookieUtil;

import java.util.function.Supplier;

@RestController
@RequestMapping(API.V1 + API.AUTH)
@RequiredArgsConstructor
public class AuthenticationController {

    @Autowired
    private final AuthenticationService authenticationService;

    @Autowired
    private final CookieUtil cookieUtil;

    //    @Autowired
    //    private final AdministratorService administratorService;
    //
    //    @PostMapping(API.REGISTER + API.ADMIN)
    //    public ResponseEntity<LoginResponse> registerAdmin(@Validated @RequestBody AdminRegisterRequest request) {
    //        return login(() -> administratorService.registerAdmin(request), HttpStatus.CREATED);
    //    }

    @PostMapping(API.LOGIN + API.ADMIN)
    public ResponseEntity<LoginResponse> loginAdmin(@Validated @RequestBody AdminLoginRequest request) {
        return login(() -> authenticationService.loginAdmin(request), HttpStatus.ACCEPTED);
    }

    @PostMapping(API.REGISTER + API.STUDENT)
    public ResponseEntity<LoginResponse> registerStudent(@Validated @RequestBody StudentRegisterRequest request) {
        return login(() -> authenticationService.registerStudent(request), HttpStatus.CREATED);
    }

    @PostMapping(API.LOGIN + API.STUDENT)
    public ResponseEntity<LoginResponse> loginStudent(@Validated @RequestBody StudentLoginRequest request) {
        return login(() -> authenticationService.loginStudent(request), HttpStatus.ACCEPTED);
    }

    @PostMapping(API.REGISTER + API.TEACHER)
    public ResponseEntity<LoginResponse> registerTeacher(@Validated @RequestBody TeacherRegisterRequest request) {
        return login(() -> authenticationService.registerTeacher(request), HttpStatus.CREATED);
    }

    @PostMapping(API.LOGIN + API.TEACHER)
    public ResponseEntity<LoginResponse> loginTeacher(@Validated @RequestBody TeacherLoginRequest request) {
        return login(() -> authenticationService.loginTeacher(request), HttpStatus.ACCEPTED);
    }

    private ResponseEntity<LoginResponse> login(Supplier<LoginResponse> supplier, HttpStatus status) {
        LoginResponse response = supplier.get();

        return ResponseEntity.status(status.value())
            .header(
                HttpHeaders.SET_COOKIE,
                cookieUtil.createCookie(
                    response.getToken(),
                    CookieUtil.COOKIE_NAME
                ).toString()
            ).body(response);
    }
}
