package studentmanagement.api.rest.controller.course;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import studentmanagement.model.API.API;
import studentmanagement.model.course.Course;
import studentmanagement.model.course.dto.CourseStudentListDto;
import studentmanagement.model.dto.create.CourseCreateRequest;
import studentmanagement.model.dto.create.CourseUpsertRequest;
import studentmanagement.model.dto.response.CourseCreateResponse;
import studentmanagement.model.key.IDableKey;
import studentmanagement.service.course.CourseService;

import java.util.List;
import java.util.function.Supplier;

@RestController
@RequestMapping(API.V1 + API.COURSE)
public class DefaultCourseController {
    private final CourseService courseService;

    @Autowired
    public DefaultCourseController(CourseService courseService) {
        this.courseService = courseService;
    }


    @PostMapping
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<CourseCreateResponse> create(@Validated @RequestBody CourseCreateRequest request) {
        return new ResponseEntity<>(courseService.create(request), HttpStatus.CREATED);
    }

    @PutMapping
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<CourseCreateResponse> upsert(@Validated @RequestBody CourseUpsertRequest request) {
        return new ResponseEntity<>(courseService.upsert(request), HttpStatus.ACCEPTED);
    }

    @GetMapping("/{id}")
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<CourseStudentListDto> findById(@Validated @PathVariable(value = "id") IDableKey courseKey) {
        return ResponseEntity.ok(courseService.findByIdMappedToDto(courseKey));
    }

    @DeleteMapping("/{id}")
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<Course> deleteById(@Validated @PathVariable(name = "id") IDableKey studentKey) {
        return ResponseEntity.ok(courseService.deleteById(studentKey));
    }

    @GetMapping
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<List<CourseStudentListDto>> findAll() {
        return supplyAndHandleListResponse(courseService::findAllMappedToDto);
    }

    @GetMapping("/sort")
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<List<CourseStudentListDto>> findAllSorted() {
        return supplyAndHandleListResponse(courseService::findAllSortedByNameMappedToDto);
    }

    private <T> ResponseEntity<List<T>> supplyAndHandleListResponse(Supplier<List<T>> supplier) {
        List<T> list = supplier.get();
        return new ResponseEntity<>(list, (list.isEmpty() ? HttpStatus.NO_CONTENT : HttpStatus.OK));
    }
}
