package studentmanagement.api.rest.controller.student;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import studentmanagement.model.API.API;
import studentmanagement.model.dto.create.StudentUpsertRequest;
import studentmanagement.model.dto.find.StudentIdNameDto;
import studentmanagement.model.dto.response.StudentUpsertResponse;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.student.Student;
import studentmanagement.service.student.StudentService;

import java.util.List;

@RestController
@RequestMapping(API.V1 + API.STUDENT)
public class DefaultStudentController {
    private final StudentService studentService;

    @Autowired
    public DefaultStudentController(StudentService studentService) {
        this.studentService = studentService;
    }

    @PutMapping
    @PreAuthorize(API.ADMIN_TEACHER_AUTHORITY)
    public ResponseEntity<StudentUpsertResponse> upsert(@Validated  @RequestBody StudentUpsertRequest request) {
        return new ResponseEntity<>(studentService.upsert(request), HttpStatus.ACCEPTED);
    }

    @GetMapping("/{id}")
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<StudentIdNameDto> findById(@Validated @PathVariable(name = "id") IDableKey studentKey) {
        return ResponseEntity.ok(studentService.findByIdMappedToDto(studentKey));
    }

    @DeleteMapping("/{id}")
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<Student> deleteById(@Validated @PathVariable(name = "id") IDableKey studentKey) {
        return ResponseEntity.ok(studentService.deleteById(studentKey));
    }

    @GetMapping
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<List<StudentIdNameDto>> findAll() {
        List<StudentIdNameDto> students = studentService.findAllMappedToDto();

        return new ResponseEntity<>(students, (students.isEmpty() ? HttpStatus.NO_CONTENT : HttpStatus.OK));
    }
}
