package studentmanagement.api.rest.controller.teacher;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import studentmanagement.model.API.API;
import studentmanagement.model.dto.create.TeacherUpsertRequest;
import studentmanagement.model.dto.response.TeacherUpsertResponse;
import studentmanagement.model.dto.find.TeacherIdNameDto;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.teacher.Teacher;
import studentmanagement.service.teacher.TeacherService;

import java.util.List;

@RestController
@RequestMapping(API.V1 + API.TEACHER)
public class DefaultTeacherController {
    private final TeacherService teacherService;

    @Autowired
    public DefaultTeacherController(TeacherService teacherService) {
        this.teacherService = teacherService;
    }

    @PutMapping
    @PreAuthorize(API.ADMIN_TEACHER_AUTHORITY)
    public ResponseEntity<TeacherUpsertResponse> upsert(@Validated @RequestBody TeacherUpsertRequest request) {
        return new ResponseEntity<>(teacherService.upsert(request), HttpStatus.ACCEPTED);
    }

    @GetMapping("/{id}")
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<TeacherIdNameDto> findById(@Validated @PathVariable(name = "id") IDableKey teacherKey) {
        return ResponseEntity.ok(teacherService.findByIdMappedToDto(teacherKey));
    }

    @DeleteMapping("/{id}")
    @PreAuthorize(API.ADMIN_AUTHORITY)
    public ResponseEntity<Teacher> deleteById(@Validated @PathVariable(name = "id") IDableKey teacherKey) {
        return ResponseEntity.ok(teacherService.deleteById(teacherKey));
    }

    @GetMapping
    @PreAuthorize(API.ANY_AUTHORITY)
    public ResponseEntity<List<TeacherIdNameDto>> findAll() {
        List<TeacherIdNameDto> teachers = teacherService.findAllMappedToDto();

        return new ResponseEntity<>(teachers, (teachers.isEmpty() ? HttpStatus.NO_CONTENT : HttpStatus.OK));
    }
}
