package studentmanagement.api.security;

import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.logout.LogoutHandler;
import org.springframework.stereotype.Service;
import studentmanagement.model.person.Person;
import studentmanagement.model.token.Token;
import studentmanagement.service.cookie.CookieUtil;
import studentmanagement.service.jwt.JwtService;
import studentmanagement.service.user_details.DefaultUserDetailsService;

import java.util.Optional;

@Service
@RequiredArgsConstructor
public class LogoutUtil implements LogoutHandler {
    @Autowired
    private final JwtService jwtService;

    @Autowired
    private final DefaultUserDetailsService userDetailsService;

    @Autowired
    private final CookieUtil cookieUtil;

    @Override
    public void logout(HttpServletRequest request, HttpServletResponse response, Authentication authentication) {
        Optional<Cookie> cookieOptional = cookieUtil.findServletCookie(request, CookieUtil.COOKIE_NAME);

        if (cookieOptional.isEmpty()) {
            return;
        }

        Cookie cookie = cookieOptional.get();

        String jwtToken = cookie.getValue();

        if (jwtToken.equals(JwtService.INVALID_TOKEN)) {
            return;
        }

        String username = jwtService.extractUsername(jwtToken);

        if (username == null || SecurityContextHolder.getContext().getAuthentication() != null) {
            return;
        }

        Person userDetails = this.userDetailsService.loadUserByUsername(username);

        Token token = jwtService.findTokenByPersonOrElse(userDetails, null);

        if (token == null) {
            return;
        }

        jwtService.delete(token);
        response.setHeader(HttpHeaders.SET_COOKIE,
            cookieUtil.createCookie(JwtService.INVALID_TOKEN, CookieUtil.COOKIE_NAME).toString());

        logDelete(token);
    }

    private void logDelete(Token token) {
        System.out.println(System.lineSeparator()
                           + System.lineSeparator()
                           + System.lineSeparator());

        System.out.println("Deleting Token for User: " + token.getPerson().getUsername());

        System.out.println(System.lineSeparator()
                           + System.lineSeparator()
                           + System.lineSeparator());
    }
}
