package studentmanagement.api.rest.administrator;

import com.google.gson.Gson;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import studentmanagement.model.API.API;
import studentmanagement.api.rest.controller.administrator.DefaultAdministratorController;
import studentmanagement.model.dto.find.StudentCourseIdRequest;
import studentmanagement.model.dto.find.TeacherCourseIdRequest;
import studentmanagement.model.dto.find.CourseIdNameDto;
import studentmanagement.model.dto.response.StudentCourseResponse;
import studentmanagement.model.dto.find.StudentIdNameDto;
import studentmanagement.model.dto.response.TeacherCourseResponse;
import studentmanagement.model.dto.find.TeacherIdNameDto;
import studentmanagement.service.administrator.AdministratorService;

import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootApplication
class DefaultAdministratorControllerTest {
    private static final String ADMIN = "/admin";
    private static final String ENROLL = "/enroll";
    private static final String DISMISS = "/dismiss";
    private static final String EMPLOY = "/employ";
    private static final String FIRE = "/fire";

    private static AdministratorService administratorService;

    private static DefaultAdministratorController controller;

    @Autowired
    private static MockMvc mockMvc;

    private Gson gson = new Gson();

    @BeforeAll
    static void setUpTestCase() {
        administratorService = Mockito.mock(AdministratorService.class);
        controller = new DefaultAdministratorController(administratorService, roleChangeService);
        mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
    }

    @Test
    void enroll_Returns_Ok() throws Exception {
        StudentCourseIdRequest request = new StudentCourseIdRequest(1L, 1L);
        StudentCourseResponse response = new StudentCourseResponse(
            new StudentIdNameDto(1L, "Filip"),
            new CourseIdNameDto(1L, "Java")
        );

        String requestJson = gson.toJson(request);
        String responseJson = gson.toJson(response);

        Mockito.when(administratorService.enroll(request)).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.patch(API.V1 + ADMIN + ENROLL)
                .contentType(MediaType.APPLICATION_JSON)
                .content(requestJson)
                .accept(MediaType.APPLICATION_JSON)
        ).andExpect(status().isOk()).andExpect(content().string(responseJson));
    }

    @Test
    void dismiss_Returns_Ok() throws Exception {
        StudentCourseIdRequest request = new StudentCourseIdRequest(1L, 1L);
        StudentCourseResponse response = new StudentCourseResponse(
            new StudentIdNameDto(1L, "Filip"),
            new CourseIdNameDto(1L, "Java")
        );

        String requestJson = gson.toJson(request);
        String responseJson = gson.toJson(response);

        Mockito.when(administratorService.dismiss(request)).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.patch(API.V1 + ADMIN + DISMISS)
                .contentType(MediaType.APPLICATION_JSON)
                .content(requestJson)
                .accept(MediaType.APPLICATION_JSON)
        ).andExpect(status().isOk()).andExpect(content().string(responseJson));
    }

    @Test
    void employ() throws Exception {
        TeacherCourseIdRequest request = new TeacherCourseIdRequest(1L, 1L);
        TeacherCourseResponse response = new TeacherCourseResponse(
            new TeacherIdNameDto(1L, "Filip"),
            new CourseIdNameDto(1L, "Java")
        );

        String requestJson = gson.toJson(request);
        String responseJson = gson.toJson(response);

        Mockito.when(administratorService.employ(request)).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.patch(API.V1 + ADMIN + EMPLOY)
                .contentType(MediaType.APPLICATION_JSON)
                .content(requestJson)
                .accept(MediaType.APPLICATION_JSON)
        ).andExpect(status().isOk()).andExpect(content().string(responseJson));
    }

    @Test
    void fire() throws Exception {
        TeacherCourseIdRequest request = new TeacherCourseIdRequest(1L, 1L);
        TeacherCourseResponse response = new TeacherCourseResponse(
            new TeacherIdNameDto(1L, "Filip"),
            new CourseIdNameDto(1L, "Java")
        );

        String requestJson = gson.toJson(request);
        String responseJson = gson.toJson(response);

        Mockito.when(administratorService.fire(request)).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.patch(API.V1 + ADMIN + FIRE)
                .contentType(MediaType.APPLICATION_JSON)
                .content(requestJson)
                .accept(MediaType.APPLICATION_JSON)
        ).andExpect(status().isOk()).andExpect(content().string(responseJson));
    }
}