package studentmanagement.api.rest.teacher;

import com.google.gson.Gson;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import studentmanagement.model.API.API;
import studentmanagement.api.rest.controller.teacher.DefaultTeacherController;
import studentmanagement.model.dto.create.TeacherRegisterRequest;
import studentmanagement.model.dto.response.TeacherUpsertResponse;
import studentmanagement.model.dto.find.TeacherIdNameDto;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.teacher.Degree;
import studentmanagement.model.person.teacher.Teacher;
import studentmanagement.service.teacher.TeacherService;

import java.util.List;

import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

class DefaultTeacherControllerTest {
    private static final String TEACHER = "/teacher";
    private final Gson gson = new Gson();
    private TeacherService teacherService;
    private DefaultTeacherController controller;
    @Autowired
    private MockMvc mockMvc;

    @BeforeEach
    void setUpTestCase() {
        teacherService = Mockito.mock(TeacherService.class);
        controller = new DefaultTeacherController(teacherService);
        mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
    }

    @Test
    void findById_Returns_200() throws Exception {
        IDableKey request = new IDableKey(1L);
        TeacherIdNameDto response = new TeacherIdNameDto(1L, "Filip");

        String responseJson = gson.toJson(response);

        Mockito.when(teacherService.findByIdMappedToDto(request)).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.get(API.V1 + TEACHER + "/{id}", 1)
        ).andExpect(status().isOk()).andExpect(content().json(responseJson));
    }

    @Test
    void deleteById_Returns_200() throws Exception {
        IDableKey request = new IDableKey(1L);
        Teacher response = new Teacher();
        response.setName("Filip");
        response.setId(1L);
        response.setDegree(Degree.BSC);

        String responseJson = gson.toJson(response);

        Mockito.when(teacherService.deleteById(request)).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.delete(API.V1 + TEACHER + "/{id}", 1)
        ).andExpect(status().isOk()).andExpect(content().json(responseJson));
    }

    @Test
    void findAll_Returns_200() throws Exception {
        TeacherIdNameDto student1 = new TeacherIdNameDto(1L, "Filip");
        TeacherIdNameDto student2 = new TeacherIdNameDto(2L, "Filip2");
        TeacherIdNameDto student3 = new TeacherIdNameDto(3L, "Filip3");
        List<TeacherIdNameDto> response = List.of(student1, student2, student3);

        String responseJson = gson.toJson(response);

        Mockito.when(teacherService.findAllMappedToDto()).thenReturn(response);

        mockMvc.perform(
            MockMvcRequestBuilders.get(API.V1 + TEACHER, 1)
        ).andExpect(status().isOk()).andExpect(content().json(responseJson));
    }

    @Test
    void upsert_Returns_200() throws Exception {
        TeacherRegisterRequest request = new TeacherRegisterRequest("Filip", "BSC");
        TeacherUpsertResponse response = new TeacherUpsertResponse(1L, "Filip", Degree.BSC);

        String requestJson = gson.toJson(request);
        String responseJson = gson.toJson(response);

        Mockito.when(teacherService.create(request)).thenReturn(response);

        mockMvc.perform(
                MockMvcRequestBuilders.put(API.V1 + TEACHER)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(requestJson)
            ).andExpect(status().isOk())
            .andExpect(content().string(responseJson));
    }
}