package studentmanagement.service.course;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import studentmanagement.data.course.CourseRepository;
import studentmanagement.mapper.course.CourseMapper;
import studentmanagement.model.course.Course;
import studentmanagement.model.course.dto.CourseStudentListDto;
import studentmanagement.model.dto.create.CourseCreateRequest;
import studentmanagement.model.dto.create.CourseUpsertRequest;
import studentmanagement.model.dto.response.CourseCreateResponse;
import studentmanagement.model.dto.find.CourseIdNameDto;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.teacher.Teacher;
import studentmanagement.service.generic.DefaultService;
import studentmanagement.service.teacher.TeacherService;

import java.util.List;

@Service
public class DefaultCourseService extends DefaultService<Course> implements CourseService {
    private final TeacherService teacherService;
    private final CourseMapper courseMapper;
    private final CourseRepository courseRepository;

    @Autowired
    public DefaultCourseService(CourseRepository courseRepository, TeacherService teacherService,
                                CourseMapper courseMapper, CourseRepository courseRepository1) {
        super(courseRepository);
        this.teacherService = teacherService;
        this.courseMapper = courseMapper;
        this.courseRepository = courseRepository1;
    }

    @Override
    public CourseCreateResponse create(CourseCreateRequest dto) {
        Teacher teacher = teacherService.findById(dto.teacherKey());

        Course course = courseCreateRequestToCourse(dto);
        course.setTeacher(teacher);

        return courseMapper.courseToCourseCreateResponse(save(course));
    }

    @Override
    public CourseCreateResponse upsert(CourseUpsertRequest dto) {
        Teacher teacher = teacherService.findById(dto.teacherKey());

        Course course = courseUpsertRequestToCourse(dto);
        course.setTeacher(teacher);

        return courseMapper.courseToCourseCreateResponse(save(course));
    }

    @Override
    public CourseStudentListDto findByIdMappedToDto(IDableKey courseKey) {
        return courseToCourseStudentListDto(findById(courseKey));
    }

    @Override
    public List<CourseStudentListDto> findAllMappedToDto() {
        return findAll().stream()
            .map(this::courseToCourseStudentListDto).toList();
    }

    @Override
    public List<CourseStudentListDto> findAllSortedByNameMappedToDto() {
        return courseRepository.findAll(Sort.by(Sort.Direction.ASC, "name")).stream()
            .map(this::courseToCourseStudentListDto)
            .toList();
    }

    private Course courseCreateRequestToCourse(CourseCreateRequest request) {
        return courseMapper.courseCreateRequestToCourse(request);
    }

    private Course courseUpsertRequestToCourse(CourseUpsertRequest request) {
        return courseMapper.courseUpsertRequestToCourse(request);
    }

    private CourseIdNameDto courseToCourseIdNameDto(Course course) {
        return courseMapper.courseToCourseIdNameDto(course);
    }

    private CourseStudentListDto courseToCourseStudentListDto(Course course) {
        return courseMapper.courseToCourseStudentListDto(course);
    }
}