package studentmanagement.service.generic;

import lombok.AllArgsConstructor;
import org.springframework.data.jpa.repository.JpaRepository;
import studentmanagement.model.IDable;
import studentmanagement.model.key.IDableKey;
import studentmanagement.service.generic.exception.IDableDoesNotExistException;

import java.util.Collection;

@AllArgsConstructor
public class DefaultService<T extends IDable> implements Service<T> {
    private static final String DOES_NOT_EXIST = "IDable with this ID does not exist";
    private static final String ALREADY_EXISTS = "IDable with this ID already exists";

    private final JpaRepository<T, Long> repository;

    @Override
    public T save(T idable) {
        return repository.save(idable);
    }

    @Override
    public T findById(IDableKey key) {
        return repository.findById(key.id())
            .orElseThrow(() -> new IDableDoesNotExistException("Idable with " + key + " does not exist"));
    }

    @Override
    public T deleteById(IDableKey key) {
        T idable = findById(key);

        repository.deleteById(key.id());

        return idable;
    }

    @Override
    public Collection<T> findAll() {
        return repository.findAll();
    }

    @Override
    public Collection<IDableKey> findAllKeys() {
        return repository.findAll().stream().map(IDable::getId).map(IDableKey::new).toList();
    }
}
