package studentmanagement.service.jwt.purger;

import lombok.RequiredArgsConstructor;
import studentmanagement.model.token.Token;
import studentmanagement.service.jwt.JwtService;

import java.util.Date;
import java.util.List;

@RequiredArgsConstructor
public class JwtTokenPurger implements Runnable {

    private final JwtService jwtService;

    private Date now;

    @Override
    public void run() {
        now = new Date();

        List<Token> tokens = jwtService.findAll().stream()
            .filter(this::filter)
            .toList();

        if (!tokens.isEmpty()) {
            jwtService.deleteAll(tokens);
            logDeletes(tokens);
        }

    }

    private void logDeletes(List<Token> tokens) {
        System.out.println(System.lineSeparator()
                           + System.lineSeparator()
                           + System.lineSeparator());
        tokens.forEach(token ->
            System.out.println("Deleting token for User: " + token.getPerson().getUsername())
        );

        System.out.println(System.lineSeparator()
                           + System.lineSeparator()
                           + System.lineSeparator());
    }

    private Boolean filter(Token token) {
        Date expirationDate = jwtService.extractExpiration(token.getToken());

        long diff = now.getTime() - expirationDate.getTime();

        return diff > 30;
    }
}
