package studentmanagement.service.role;

import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import studentmanagement.model.dto.create.RoleChangeRequest;
import studentmanagement.model.dto.response.RoleChangeResponse;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.Person;
import studentmanagement.model.role.Role;
import studentmanagement.service.administrator.AdministratorService;
import studentmanagement.service.role.exception.InvalidRoleException;
import studentmanagement.service.student.StudentService;
import studentmanagement.service.teacher.TeacherService;
import studentmanagement.service.user_details.DefaultUserDetailsService;

import java.util.function.Function;

@AllArgsConstructor
@Service
public class RoleChangeService {

    @Autowired
    private final DefaultUserDetailsService userDetailsService;

    @Autowired
    private final StudentService studentService;

    @Autowired
    private final TeacherService teacherService;

    @Autowired
    private final AdministratorService administratorService;


    //    @Transactional
    // Can not be transactional because the unique constraint of the username stops the creating of a new entity
    public RoleChangeResponse changeRole(RoleChangeRequest request) {
        Person person = userDetailsService.loadUserByUsername(request.getUsername());
        IDableKey key = new IDableKey(person.getId());

        Person deleted = deleteFromCurrentRole(
            key,
            request.getCurrentRole().toUpperCase()
        ).apply(person);

        Person created = createToNewRole(
            request.getNewRole().toUpperCase()
        ).apply(person);

        return RoleChangeResponse.builder()
            .id(created.getId())
            .name(created.getName())
            .username(created.getUsername())
            .build();
    }


    private Function<Person, Person> deleteFromCurrentRole(IDableKey key, String currentRole) {
        return (person) -> {
            if (currentRole.equals(Role.ADMIN.getAuthority())) {
                return administratorService.deleteById(key);
            }

            if (currentRole.equals(Role.TEACHER.getAuthority())) {
                return teacherService.deleteById(key);
            }

            if (currentRole.equals(Role.STUDENT.getAuthority())) {
                return studentService.deleteById(key);
            }

            throw new InvalidRoleException("Error while determining Role of Person");
        };
    }

    private Function<Person, Person> createToNewRole(String newRole) {
        return (person) -> {
            if (newRole.equals(Role.ADMIN.getAuthority())) {
                return administratorService.create(person);
            }

            if (newRole.equals(Role.TEACHER.getAuthority())) {
                return teacherService.create(person);
            }

            if (newRole.equals(Role.STUDENT.getAuthority())) {
                return studentService.create(person);
            }

            throw new InvalidRoleException("Error while determining Role of Person");
        };
    }
}
