package studentmanagement.service.user_details;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import studentmanagement.model.person.Person;
import studentmanagement.model.person.admin.Admin;
import studentmanagement.model.person.student.Student;
import studentmanagement.model.person.teacher.Teacher;
import studentmanagement.service.administrator.AdministratorService;
import studentmanagement.service.student.StudentService;
import studentmanagement.service.teacher.TeacherService;

import java.util.Objects;

@Service
public class DefaultUserDetailsService implements UserDetailsService {

    private final StudentService studentService;
    private final TeacherService teacherService;
    private final AdministratorService adminService;

    @Autowired
    public DefaultUserDetailsService(StudentService studentService, TeacherService teacherService,
                                     AdministratorService adminService) {
        this.studentService = studentService;
        this.teacherService = teacherService;
        this.adminService = adminService;
    }

    @Override
    public Person loadUserByUsername(String username) throws UsernameNotFoundException {
        Student student = studentService
            .findByUsernameOrElse(username, null);

        Teacher teacher = teacherService
            .findByUsernameOrElse(username, null);

        Admin administrator = adminService
            .findByUsernameOrElse(username, null);

        return Objects.requireNonNullElseGet(student, () -> Objects.requireNonNullElse(teacher, administrator));
    }
}
