package studentmanagement.service.course;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import studentmanagement.data.course.CourseRepository;
import studentmanagement.model.course.Course;
import studentmanagement.model.dto.create.CourseCreateRequest;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.teacher.Degree;
import studentmanagement.model.person.teacher.Teacher;
import studentmanagement.service.generic.exception.IDableDoesNotExistException;
import studentmanagement.service.teacher.TeacherService;

class DefaultCourseServiceTest {
    private CourseRepository mockCourseRepo;
    private CourseService courseService;
    private TeacherService mockTeacherService;

    @BeforeEach
    void setUpTestCase() {
        mockTeacherService = Mockito.mock(TeacherService.class);
        mockCourseRepo = Mockito.mock(CourseRepository.class);
        courseService = new DefaultCourseService(mockCourseRepo, mockTeacherService, courseMapper, courseRepository1);
    }

    @Test
    void add_Adds_Course_Correctly() throws IDableDoesNotExistException {
        long teacherId = 0;
        String teacherIdString = "0";
        String teacherName = "Filip";
        Degree degree = Degree.PHD;

        Teacher teacher = new Teacher(teacherName, degree);
        teacher.setId(teacherId);
        Mockito.when(mockTeacherService.findById(new IDableKey(teacherIdString))).thenReturn(teacher);

        long courseId = 1;
        String courseIdString = "1";
        String courseName = "Java";
        long totalHours = 150;
        String totalHoursString = "150";

        Course course = new Course(courseName, totalHours, teacher);
        course.setId(courseId);

        Mockito.when(mockCourseRepo.saveAndFlush(Mockito.any())).thenReturn(course);

        Course actual =
            courseService.create(new CourseCreateRequest(courseName, totalHoursString, new IDableKey(teacherIdString)));

        Assertions.assertEquals(course, actual, "Course was not created and added correctly");
    }
}