package studentmanagement.service.generic;


import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import studentmanagement.data.student.StudentRepository;
import studentmanagement.model.key.IDableKey;
import studentmanagement.model.person.student.Student;
import studentmanagement.service.generic.exception.IDableDoesNotExistException;

import java.util.Optional;

class DefaultServiceTest {

    private final long id = 1;
    private final IDableKey key = new IDableKey(id);
    private final String name = "Filip";
    private final long age = 123;
    private final Student student = new Student(name, age);
    private StudentRepository mockStudentRepo;
    private Service<Student> studentService;

    @BeforeEach
    void setUpTestCase() {
        student.setId(id);
        mockStudentRepo = Mockito.mock(StudentRepository.class);
        studentService = new DefaultService<>(mockStudentRepo);
    }

    @Test
    void add_Adds_Object_Correctly() {
        Mockito.when(mockStudentRepo.saveAndFlush(student)).thenReturn(student);

        Student actual = studentService.save(student);

        Assertions.assertEquals(student, actual, "Student was not added correctly");
    }

    @Test
    void get_Returns_Object_Correctly() throws IDableDoesNotExistException {
        Optional<Student> optionalStudent = Optional.of(student);
        Mockito.when(mockStudentRepo.findById(key.id())).thenReturn(optionalStudent);

        Student actual = studentService.findById(key);

        Assertions.assertEquals(student, actual, "Student was not found correctly");
    }

    @Test
    void get_Throws_IDableDoesNotExistException() {
        Optional<Student> optionalStudent = Optional.empty();
        Mockito.when(mockStudentRepo.findById(key.id())).thenReturn(optionalStudent);

        Assertions.assertThrows(IDableDoesNotExistException.class, () -> studentService.findById(key),
            "IDableDoesNotExistException expected");
    }


    @Test
    void remove_Removes_Object_Correctly() throws IDableDoesNotExistException {
        Optional<Student> optionalStudent = Optional.of(student);
        Mockito.when(mockStudentRepo.findById(key.id())).thenReturn(optionalStudent);

        Student actual = studentService.deleteById(key);

        Assertions.assertEquals(student, actual, "Student was not removed correctly");
    }

    @Test
    void remove_Throws_IDableDoesNotExistException() {
        Optional<Student> optionalStudent = Optional.empty();
        Mockito.when(mockStudentRepo.findById(key.id())).thenReturn(optionalStudent);

        Assertions.assertThrows(IDableDoesNotExistException.class, () -> studentService.deleteById(key),
            "IDableDoesNotExistException expected");
    }
}