package studentmanagement.data.grade;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import studentmanagement.model.course.Course;
import studentmanagement.model.grade.Grade;
import studentmanagement.model.person.student.Student;

import java.util.List;
import java.util.Optional;

@Repository
public interface GradeRepository extends JpaRepository<Grade, Long> {
    List<Grade> findByStudentId(Long studentId);

    List<Grade> findByCourseId(Long courseId);

    List<Grade> findAllByStudentIdAndCourseId(Long studentId, Long courseId);

    @Query("SELECT AVG(g.grade) FROM Grade g WHERE g.student = :student")
    Optional<Double> findTotalAverageGradeForStudent(@Param("student") Student student);

    @Query("SELECT AVG(g.grade) FROM Grade g WHERE g.course = :course")
    Optional<Double> findTotalAverageGradeForCourse(@Param("course") Course course);

    @Query("SELECT AVG(g.grade) FROM Grade g WHERE g.student = :student AND g.course = :course")
    Optional<Double> findAverageGradeForStudentInCourse(@Param("student") Student student,
                                                        @Param("course") Course course);
}