package studentmanagement.mapper.admin;

import org.mapstruct.Context;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import studentmanagement.model.course.Course;
import studentmanagement.model.dto.create.AdminRegisterRequest;
import studentmanagement.model.dto.create.AdminUpsertRequest;
import studentmanagement.model.dto.find.AdminIdNameDto;
import studentmanagement.model.dto.response.AdminUpsertResponse;
import studentmanagement.model.dto.response.StudentCourseResponse;
import studentmanagement.model.dto.response.TeacherCourseResponse;
import studentmanagement.model.person.Person;
import studentmanagement.model.person.admin.Admin;
import studentmanagement.model.person.student.Student;
import studentmanagement.model.person.teacher.Teacher;

import java.util.List;

@Mapper(componentModel = "spring")
public interface AdminMapper {

    @Mapping(target = "id", ignore = true)
    @Mapping(target = "password", source = "password", qualifiedByName = "encodePassword")
    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "token", ignore = true)
    Admin adminRegisterRequestToAdmin(AdminRegisterRequest request, @Context BCryptPasswordEncoder passwordEncoder);

    @Named("encodePassword")
    default String encodePassword(String password, @Context BCryptPasswordEncoder passwordEncoder) {
        return passwordEncoder.encode(password);
    }

    AdminIdNameDto adminToAdminIdNameDto(Admin admin);

    List<AdminIdNameDto> allAdminsToAdminIdNameDto(List<Admin> admins);

    AdminUpsertResponse adminToAdminCreateResponse(Admin admin);

    @Mapping(source = "student", target = "student")
    @Mapping(source = "course", target = "course")
    StudentCourseResponse newStudentCourseResponse(Student student, Course course);

    @Mapping(source = "teacher", target = "teacher")
    @Mapping(source = "course", target = "course")
    TeacherCourseResponse newTeacherCourseResponse(Teacher teacher, Course course);

    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "token", ignore = true)
    @Mapping(target = "password", source = "password", qualifiedByName = "encodePassword")
    Admin adminUpsertRequestToAdmin(AdminUpsertRequest request,
                                    @Context BCryptPasswordEncoder passwordEncoder);

    @Mapping(target = "id", ignore = true)
    @Mapping(target = "authority", ignore = true)
    @Mapping(target = "authorities", ignore = true)
    Admin personToAdmin(Person person);
}
