package studentmanagement.mapper.student;

import org.mapstruct.Context;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import studentmanagement.model.dto.create.StudentRegisterRequest;
import studentmanagement.model.dto.create.StudentUpsertRequest;
import studentmanagement.model.dto.find.StudentIdNameDto;
import studentmanagement.model.dto.response.StudentUpsertResponse;
import studentmanagement.model.person.Person;
import studentmanagement.model.person.student.Student;

@Mapper(componentModel = "spring")
public interface StudentMapper {

    @Mapping(target = "id", ignore = true)
    @Mapping(target = "courses", ignore = true)
    @Mapping(target = "password", source = "password", qualifiedByName = "encodePassword")
    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "token", ignore = true)
    Student studentCreateRequestToStudent(StudentRegisterRequest request,
                                          @Context BCryptPasswordEncoder passwordEncoder);

    @Named(value = "encodePassword")
    default String encodePassword(String password, @Context BCryptPasswordEncoder passwordEncoder) {
        return passwordEncoder.encode(password);
    }

    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "token", ignore = true)
    @Mapping(target = "courses", ignore = true)
    @Mapping(target = "password", source = "password", qualifiedByName = "encodePassword")
    Student studentUpsertRequestToStudent(StudentUpsertRequest request,
                                          @Context BCryptPasswordEncoder passwordEncoder);

    StudentIdNameDto studentToStudentIdNameDto(Student student);

    StudentUpsertResponse studentToStudentCreateResponse(Student student);

    @Mapping(target = "id", ignore = true)
    @Mapping(target = "authority", ignore = true)
    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "age", ignore = true)
    @Mapping(target = "courses", ignore = true)
    Student personToStudent(Person person);
}
