package studentmanagement.mapper.teacher;

import org.mapstruct.Context;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import studentmanagement.model.dto.create.TeacherRegisterRequest;
import studentmanagement.model.dto.create.TeacherUpsertRequest;
import studentmanagement.model.dto.find.TeacherIdNameDto;
import studentmanagement.model.dto.response.TeacherUpsertResponse;
import studentmanagement.model.person.Person;
import studentmanagement.model.person.teacher.Teacher;

@Mapper(componentModel = "spring")
public interface TeacherMapper {

    @Mapping(target = "id", ignore = true)
    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "token", ignore = true)
    @Mapping(target = "password", source = "password", qualifiedByName = "encodePassword")
    Teacher teacherRegisterRequestToTeacher(TeacherRegisterRequest request,
                                            @Context BCryptPasswordEncoder passwordEncoder);

    @Named("encodePassword")
    default String encodePassword(String password, @Context BCryptPasswordEncoder passwordEncoder) {
        return passwordEncoder.encode(password);
    }

    @Mapping(target = "id", ignore = true)
    @Mapping(target = "authority", ignore = true)
    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "degree", ignore = true)
    Teacher personToTeacher(Person person);

    @Mapping(target = "authorities", ignore = true)
    @Mapping(target = "token", ignore = true)
    @Mapping(target = "password", source = "password", qualifiedByName = "encodePassword")
    Teacher teacherUpsertRequestToTeacher(TeacherUpsertRequest request, @Context BCryptPasswordEncoder passwordEncoder);

    TeacherIdNameDto teacherToTeacherIdNameDto(Teacher teacher);

    TeacherUpsertResponse teacherToTeacherCreateResponse(Teacher teacher);
}
